/*
**	gps.c
**  (c)Huron Net Works, Inc., 2005
**
**	9/6/05	-- Original
**	9/16/05	-- Add transmit capability
**
**  TABS 5, 9
*/
#include	"hardware.h"
#include	"externs.h"

/*	Configure for 38,400 baud, 8N1, and TX and Rx Interrupts	*/
/*	The following expression should compute to 0xCC09			*/

#define	GPS_UART_CONFIGURATION	\
(		SPI_UART_WRITE_CONFIGURATION	\
	+	SPI_UART_TM_BAR	\
	+	SPI_UART_RM_BAR	\
	+	BR038P4	\
)

#define	GPS_TIMEOUT		60

unsigned char	gps_state ;
unsigned char	gps_commas ;

const unsigned char	gps_header[] = "GPRMC" ;
unsigned char	sx ;	/* String Index	*/
unsigned char	bx ;	/* Buffer Index	*/
unsigned char	fx ;	/* Field Index	*/

void	gps_Init(void)
{
	/*	Called with interrupts disabled		*/
	GPSU_CS = 0 ;		/* Take GPS UART Chip Select Active		*/
	
	SPDAT = GPS_UART_CONFIGURATION/256 ;
	while((SPSTA & SPIF) == 0) ;
	SPI_IN[0] = SPDAT ;
	
	SPDAT = GPS_UART_CONFIGURATION%256 ;
	while((SPSTA & SPIF) == 0) ;
	SPI_IN[1] = SPDAT ;

	GPSU_CS = 1 ;		/* Take GPS UART Chip Select Inactive	*/

	gps_state	= 0 ;
	gps_Timer	= GPS_TIMEOUT ;
	sx = 0 ; bx = 0 ;
}
/*
** If we loose track of where we are we need to invalidate the GPS Data
*/
void gps_Invalid(void)
{
	g_latitude[0] = '-' ; g_latitude[1] = '-' ;	
	g_latitude[2] = '-' ; g_latitude[3] = '-' ;	
	g_latitude[4] = '.' ;
	g_latitude[5] = '-' ; g_latitude[6] = '-' ; g_latitude[7] = '-' ; g_latitude[8] = '-' ;
	g_latitude[9] = '\0' ;		/* Null Termination	*/
	g_latitudeIndicator = '-' ;
	
	g_longitude[0] = '-' ; g_longitude[1] = '-' ; g_longitude[2] = '-' ;
	g_longitude[3] = '-' ; g_longitude[4] = '-' ;
	g_longitude[5] = '.' ;
	g_longitude[6] = '-' ; g_longitude[7] = '-' ; g_longitude[8] = '-' ; g_longitude[9] = '-' ;
	g_longitude[10] = '\0' ;	/* Null Termination	*/
	g_longitudeIndicator = '-' ;
}
/*
**	External Interrupt 1 - EX1
*/
interrupt [0x0013] using [3] void IE1_int(void)
{
	do
	{
		GPSU_CS = 0 ;		/* Activate UART Chip Select					*/
							/* A Data Read is required to clear the interrupt	*/
				
		SPDAT = SPI_UART_READ/256 ;		/* Send 1st byte of Transaction		*/
		while((SPSTA & SPIF) == 0) ;	/* Wait for completion				*/
		SPI_IN[0] = SPDAT ;				/* Read the Result					*/
				
		SPDAT = SPI_UART_READ%256 ;		/* Send 2nd byte of Transaction		*/ 
		while((SPSTA & SPIF) == 0) ;	/* Wait for completion				*/
		SPI_IN[1] = SPDAT ;				/* Read the data character			*/
				
		GPSU_CS = 1 ;			/* Deactivate UART Chip Select				*/
				

		if(SPI_IN[0] & SPI_UART_R)	/* Valid Character	*/
		{
			if(gps_nRx < 255)		/* Check to see if there is room					*/
			{
				gps_RxBuffer[gps_RxPut] = SPI_IN[1] ;
				++gps_RxPut ;
				++gps_nRx ;
			}
			else	/* One space remains	*/
			{
				gps_RxBuffer[gps_RxPut] = '%' ;			/* Put overrun character in buffer	*/
			}
		}
		if(SPI_IN[0] & SPI_UART_T)
		{
			if(gps_nTx > 0)	/* Are there characters to send?		*/
			{
				GPSU_CS = 0 ;	/* Activate GPS UART Chip Select	*/
			
				SPDAT = SPI_UART_WRITE/256 ;		/* Send 1st byte of Transaction	*/
				while((SPSTA & SPIF) == 0) ;		/* Wait for commpletion			*/
				SPI_IN[0] = SPDAT ;					/* Consume the data				*/
			
				SPDAT = gps_TxBuffer[gps_TxGet++] ;	/* Send 2nd byte of Transaction	*/
				gps_nTx = ((--gps_nTx) & (GPS_TX_SIZE - 1)) ;
				while((SPSTA & SPIF) == 0) ;		/* Wait for completion			*/
				SPI_IN[1] = SPDAT ;					/* Consume the data				*/
			
				GPSU_CS = 1 ;	/* Deactivate GPS UART Chip Select	*/
			
				if(SPI_IN[0] & SPI_UART_R)	/* Check for Additional RX characters	*/
				{
					if(gps_nRx < 255)		/* Check to see if there is room					*/
					{
						gps_RxBuffer[gps_RxPut] = SPI_IN[1] ;
						++gps_RxPut ;
						++gps_nRx ;
					}
					else	/* One space remains	*/
					{
						gps_RxBuffer[gps_RxPut] = '%' ;			/* Put overrun character in buffer	*/
					}
				}
			}
			else
			{
				gpsTDRE = 1 ;
			}
		}
	} while (GPSU_INT == 0) ;
}

unsigned char	gps_RxDeQueue(void)
{
	EX1 = 0 ;
	if(gps_nRx > 0)	/* Check to see it there are any characters	*/
	{
		--gps_nRx ;
		B = gps_RxBuffer[gps_RxGet++] ;
	}
	else
	{
		B = '\x80' ;
	}
	EX1 = 1 ;
	return B ;
}

#define	__GPS_TXENQUEUE_REQUIRED__
#if	defined(__GPS_TXENQUEUE_REQUIRED__)
/*
**	gps_TxEnQueue
**	This routine has to manage both external interrupt enable bits because both
**	the gps UART and the gsm UART use the SPI port to move data in and out
*/
void	gps_TxEnQueue(unsigned char c)
{
	if(gps_nTx < GPS_TX_SIZE)	/* Check to see if there is room						*/
	{
		putch(c) ;
		EX1 = 0 ;	/* Disable GPS Interrupts									*/
		gps_TxBuffer[gps_TxPut++] = c ;
		gps_nTx = ((++gps_nTx) & (GPS_TX_SIZE - 1)) ;
		if(gpsTDRE == 1)	/* Check to see if transmitter is running			*/
		{
			gpsTDRE = 0 ;	/* Reset the transmitter data register empty bit	*/
			EX0 = 0 ;		/* Disable GSM Interrupts							*/
			GPSU_CS = 0 ;	/* Activate GPS UART Chip Select					*/
			
			SPDAT = SPI_UART_WRITE/256 ;	/* Send 1st byte of Transaction		*/
			while((SPSTA & SPIF) == 0) ;	/* Wait for completion				*/
			SPI_IN[0] = SPDAT ;				/* Consume the data					*/
			
			SPDAT = gps_TxBuffer[gps_TxGet++] ;	/* Send 2nd byte of Transaction	*/
			gps_nTx = ((--gps_nTx) & (GPS_TX_SIZE - 1)) ;
			while((SPSTA & SPIF) == 0) ;	/* Wait for completion				*/
			SPI_IN[1] = SPDAT ;				/* Consume the data					*/

			GPSU_CS = 1 ;	/* Deactivate GPS UART Chip Select					*/
			EX0 = 1 ;		/* Enable GSM Interrupts							*/
		}
		else
		{
		}
		EX1 = 1 ;	/* Enable GPS Interrupts									*/
	}
	else	return ;
}
#endif
unsigned char	px ;

const unsigned char	psrf_DisableGGA[] = "$PSRF103,00,00,00,01*24\r\n" ;
void	gps_sendDisableGGA(void)
{
	px = 0 ;
	while(psrf_DisableGGA[px] != '\0')	gps_TxEnQueue(psrf_DisableGGA[px++]) ;
}
const unsigned char psrf_DisableGLL[] =	"$PSRF103,01,00,00,01*25\r\n" ;
void	gps_sendDisableGLL(void)
{
	px = 0 ;
	while(psrf_DisableGLL[px] != '\0')	gps_TxEnQueue(psrf_DisableGLL[px++]) ;
}
const unsigned char	psrf_DisableGSA[] = "$PSRF103,02,00,00,01*26\r\n" ;
void	gps_sendDisableGSA(void)
{
	px = 0 ;
	while(psrf_DisableGSA[px] != '\0')	gps_TxEnQueue(psrf_DisableGSA[px++]) ;
}
const unsigned char	psrf_DisableGSV[] = "$PSRF103,03,00,00,01*27\r\n" ;
void	gps_sendDisableGSV(void)
{
	px = 0 ;
	while(psrf_DisableGSV[px] != '\0')	gps_TxEnQueue(psrf_DisableGSV[px++]) ;
}

#if	defined(__NEED_DISABLEVTG__)
const unsigned char psrf_DisableVTG[] = "$PSRF103,05,00,00,01*21\r\n" ;
void	gps_sendDisableVTG(void)
{
	px = 0 ;
	while(psrf_DisableVTG[px] != '\0')	gps_TxEnQueue(psrf_DisableVTG[px++]) ;
}		
#endif

void	gps_Parser(unsigned char c)
{
	switch(gps_state)
	{
		case 0:		/* Recognize a $			*/
			if(c == '$')
			{
				g_ComputedChecksum = 0 ;	/* Start the accumulated checksum	*/
				GPRMC[bx++] = c ;
				gps_state = 1 ;
			}
			else
			{
				bx = 0 ;
			}
			sx = 0 ;
			gps_commas = 0 ;
			break ;	
		
		case 1 :	/* Recognize the header		*/
			if(c == gps_header[sx])
			{	/* Buffer the character and advance the indexes	*/
				g_ComputedChecksum ^= c ;
				GPRMC[bx++] = c ;
				++sx ;
				if(gps_header[sx] == '\0')
				{
					gps_state = 2 ;
				}
			}
			else
			{
				gps_state = 0 ;
				sx = bx = 0 ;
			}
			gps_commas = 0 ;
			break ;
		case 2 :	/* Recognize a ','	*/
			if(c == ',')
			{
				g_ComputedChecksum ^= c ;
				GPRMC[bx++] = c ;
				++gps_commas ;
				gps_state = 3 ;
			}
			else
			{
				gps_state = 0 ;
				sx = bx = 0 ;
			}
			break ;
		case 3 : /* Alphanumeric field	*/
			if(c == ',')
			{
				g_ComputedChecksum ^= c ;
				GPRMC[bx++] = c ;
				++gps_commas ;
				fx = 0 ;
			}
			else if(c == '*')
			{
				GPRMC[bx++] = c ;
				gps_state = 4 ;
			}
			else
			{
				g_ComputedChecksum ^= c ;
				GPRMC[bx++] = c ;
				if(gps_commas == 1)
				{
					gps_Clock[fx++] = c ;
					gps_Clock[fx] = '\0' ;
				}
				else if(gps_commas == 2)	/*	GPS Status	*/
				{
					if((g_gpsStatus != 'A') && (c == 'A'))
					{
						gpsVALID = 1 ;	
					}
					g_gpsStatus = c ;
				}
				else if(gps_commas == 3)
				{
					g_latitude[fx++] = c ;
					g_latitude[fx] = '\0' ;
				}
				else if(gps_commas == 4)
				{
					g_latitudeIndicator = c ;
				}
				else if(gps_commas == 5)
				{
					g_longitude[fx++] = c ;
					g_longitude[fx] = '\0' ;
				}
				else if(gps_commas == 6)
				{
					g_longitudeIndicator = c ;
				}
			}
			break ;
		case 4 :
			GPRMC[bx++] = c ;
			g_MessageChecksum = 0 ;
			if((c >= '0') && (c <= '9')) 		g_MessageChecksum = (c -'0') ;
			else if((c >= 'A') && (c <= 'F'))	g_MessageChecksum = (10 + (c - 'A')) ;
			else if((c >= 'a') && (c <= 'f'))	g_MessageChecksum = (10 + (c - 'a')) ;
			gps_state = 5 ;
			g_MessageChecksum <<= 4 ;
			break ;
		case 5 :
			GPRMC[bx++] = c ;
			if((c >= '0') && (c <= '9')) 		g_MessageChecksum += (c -'0') ;
			else if((c >= 'A') && (c <= 'F'))	g_MessageChecksum += (10 + (c - 'A')) ;
			else if((c >= 'a') && (c <= 'f'))	g_MessageChecksum += (10 + (c - 'a')) ;
			gps_state = 6 ;
			break ;
		case 6 :
			if(c == '\r')
			{
				GPRMC[bx++] = c ;
				gps_state = 7 ;
			}
			break ;
		case 7 :
			if(c == '\n')
			{
				GPRMC[bx++] = c ;
				GPRMC[bx] = '\0' ;
				RMC = 1 ;
				gps_state = 0 ;
				bx = 0 ;
				sx = 0 ;
			}
			break ;
		default :
			break ;
	}
}

void	gpsMonitor(void)
{
	if(gps_nRx > 0)
	{
		gps_Timer = GPS_TIMEOUT ;
		if(gpsEcho == 1)
		{
			putch(gps_RxDeQueue()) ;
		}
		else
		{
			if(RMC == 0)
			{
				gps_Parser(gps_RxDeQueue()) ;
			}
		}
	}
	else
	{
		if(gps_Timer == 0)
		{
			gps_Timer = GPS_TIMEOUT ;
			GPS_RESET = 0 ;	/* 2 microsecond Reset Pulse	*/
			GPS_RESET = 0 ;
			GPS_RESET = 0 ;
			GPS_RESET = 0 ;
			GPS_RESET = 1 ;
		}	
	}
}
