/*
**  hardware.h
**  (c)Huron Net Works, Inc., 2001-2005
**
**  AT89C51xD2 Definitions
**
**	5 Jul 2005 --	Add Serial Port Definitions
**	1 Jul 2005 --	Original
**
**  TABS 5, 9
*/

#include "ioed2.h"
/*
**	Processor Hardware Definitions
*/
#define	PIEZO_OUT		P1.4
#define	P_BUTTON		P1.3
#define	LCD_REQ			P1.2
#define	USB_INT			P1.1
#define	LCD_CLK			P1.0

#define	USB_CS			P2.7
#define	LCD_CS			P2.6
#define	GSMU_CS			P2.5
#define	GPSU_CS			P2.4
#define	CHARGER			P2.3
#define	_12V0_SHDN		P2.2
#define	USB_3V3_SHDN	P2.1
#define	GPS_3V3_SHDN	P2.0

#define	A0				P3.5
#define	INVALID			P3.4
#define	GPSU_INT		P3.3
#define	GSMU_INT		P3.2

#define	REDC			P4.7	/* Bi-color LED, RED Cathode	*/
#define	GRNC			P4.6	/* Bi-color LED, GREEN Cathode	*/
#define	RING			P4.5	/* Ring Vibrator				*/
#define	LEFT			P4.4	/* LEFT Key						*/
#define	RIGHT			P4.3	/* RIGHT Key					*/
#define	DOWN			P4.2	/* DOWN Key						*/
#define	UP				P4.1	/* UP Key						*/
#define	ENTER			P4.0	/* ENTER Key					*/

#define	USB_RESET		P5.7
#define	LCD_RESET		P5.6
#define	GSM_IGT			P5.5
#define	GPS_RESET		P5.4
#define	SPKR_SHDN		P5.3
#define	GSMU_SHDN		P5.2
#define	GPSU_SHDN		P5.1
#define	RS232_SHDN		P5.0

/*
**	Bit definitions for PCON (0x87)
**	This register is not bit-addressable
**	
*/
#define	SMOD1	(1<<7)
#define	SMOD0	(1<<6)
#define	POF		(1<<4)
#define	GF1		(1<<3)
#define	GF0		(1<<2)
#define	PD		(1<<1)
#define	IDL		(1<<0)

/*
**	Serial Configuration Settings
*/
#define	SHIFT_REGISTER	(0<<6)
#define	UART8_VARIABLE	(1<<6)
#define	UART9_FIXED		(2<<6)
#define	UART9_VARIABLE	(3<<6)

/*
**	CCON Bit Definitions for constructing masks
**	CCON is a bit Addressable Register
*/
#define	CRBIT		(1<<6)
#define	CCF4BIT		(1<<4)
#define	CCF3BIT		(1<<3)
#define	CCF2BIT		(1<<2)
#define	CCF1BIT		(1<<1)
#define	CCF0BIT		(1<<0)

/*
**	IEN1 Bit Definitions
*/
#define	ESPI	(1<<2)
#define	KBD		(1<<0)
/*
**	T2MOD (0xC9) Bit Definitions
*/
#define	T2OE		(1<<1)
#define	DCEN		(1<<0)

/*
**	SPI Bit Definitions
**	SPCON (0xC3)
**	Reset Value : 0001 0100
*/
#define	SPR2	(1<<7)
#define	SPEN	(1<<6)
#define	SSDIS	(1<<5)
#define	MSTR	(1<<4)
#define	CPOL	(1<<3)
#define	CPHA	(1<<2)
#define	SPR1	(1<<1)
#define	SPR0	(1<<0)
/*
**	SPI Clock Rate Definitions
*/
#define	SP06M000	(  0  +   0  +   0)
#define	SP03M000	(  0  +   0  + SPR0)
#define	SP01M500	(  0  + SPR1 +   0 )
#define	SP750K000	(  0  + SPR1 + SPR0)
#define	SP375K000	(SPR2 +   0  +   0 )
#define	SP187K500	(SPR2 +   0  + SPR0)
#define	SP093K750	(SPR2 + SPR1 +   0 )
#define	SPNOBRG		(SPR2 + SPR1 + SPR0)
/*
**	SPI Bit Definitions
**	SPSTA (0xC4)
**	Reset Value : 00x0 xxxx
*/
#define	SPIF	(1<<7)
#define	WCOL	(1<<6)
#define	SSERR	(1<<5)
#define	MODF	(1<<4)

/*
**	Keyboard Interrupt Bits
**	P1.3 is the Panic Button
**	Applies to:
**	KBLS @ 0x9C
**	KBE  @ 0x9D
**	KBF  @ 0x9E
*/
#define	PANIC	(1<<3)
#define	KEY_DEBOUNCE		18
#define	KEY_REPEAT_INITIAL	400
#define	KEY_REPEAT			250			
/*
**	MAX3100 UART Definitions
*/
#define	SPI_UART_WRITE_CONFIGURATION	0xC000
#define	SPI_UART_READ_CONFIGURATION		0x4000
#define	SPI_UART_WRITE					0x8000
#define	SPI_UART_READ					0x0000

#define	SPI_UART_FEN_BAR				(1<<13)
#define	SPI_UART_SHDN					(1<<12)
#define	SPI_UART_TM_BAR					(1<<11)
#define	SPI_UART_RM_BAR					(1<<10)
#define	SPI_UART_PM_BAR					(1<<9)
#define	SPI_UART_RAM_BAR				(1<<8)
#define	SPI_UART_IR						(1<<7)
#define	SPI_UART_ST						(1<<6)
#define	SPI_UART_PE						(1<<5)
#define	SPI_UART_L						(1<<4)

/*
**	#define either
**	UART_CLOCK_3P6864 if the UART crystal is 3.6864 Mhz.
**	UART_CLOCK_1.8432 if the UART crystal is 1.8432 Mhz.	
*/
#define	UART_CLOCK_3P6864

#if		defined(UART_CLOCK_3P6864)

#define	BR230P4		0
#define	BR115P2		1
#define	BR057P6		2
#define	BR028P8		3
#define	BR014P4		4
#define	BR007P2		5
#define	BR003P6		6
#define	BR001P8		7
#define	BR076P8		8
#define	BR038P4		9
#define	BR019P2		10
#define	BR009P6		11
#define	BR004P8		12
#define	BR002P4		13
#define	BR001P2		14
#define	BR000P6		15

#elif	defined(UART_CLOCK_1P8432)

#define	BR115P2		0
#define	BR057P6		1
#define	BR028P8		2
#define	BR014P4		3
#define	BR007P2		4
#define	BR003P6		5
#define	BR001P8		6
#define	BR000P9		7
#define	BR038P4		8
#define	BR019P2		9
#define	BR009P6		10
#define	BR004P8		11
#define	BR002P4		12
#define	BR001P2		13
#define	BR000P6		14
#define	BR000P3		15

#else

#endif



#define	SPI_UART_R	(1<<7)
#define	SPI_UART_T	(1<<6)

/*
**	External Hardware Definitions
**	LM3097 Display/NJU6580
*/
#define	DISPLAY_OFF					0xAE
#define	DISPLAY_ON					0xAF

#define	DISPLAY_PAGE_ADDRESS		0xB0

#define	DISPLAY_COLUMN_H			0x10
#define	DISPLAY_COLUMN_L			0x00
/*
**	ADC Normal   puts column 0 on the right hand side of the glass
**	ADC Inverse  puts column 0 on the left  hand side of the glass
*/
#define	DISPLAY_ADC_NORMAL			0xA0
#define	DISPLAY_ADC_INVERSE			0xA1

#define	DISPLAY_NORMAL				0xA6
#define	DISPLAY_INVERSE				0xA7

#define	DISPLAY_ALL_NORMAL			0xA4
#define	DISPLAY_ALL_ON				0xA5

#define	DISPLAY_NO_ICON				0xAA
#define	DISPLAY_WITH_ICON			0xAB

#define	DISPLAY_READ_MODIFY_WRITE	0xE0

#define	DISPLAY_END					0xEE

#define	DISPLAY_RESET				0xE2

#define	DISPLAY_COM_SCROLL			0xC0

#define	DISPLAY_INTERNAL_PS_OFF		0x24
#define	DISPLAY_INTERNAL_PS_ON		0x25

#define	DISPLAY_VOLTAGE				0xED

#define	DISPLAY_EVR					0x80

#define	DISPLAY_POWER_SAVE_1		0xAE
#define	DISPLAY_POWER_SAVE_2		0xA5

#define	DISPLAY_SCROLL_PAGE			0x40

#define	DISPLAY_SCROLL_OFF			0xA8
#define	DISPLAY_SCROLL_ON			0xA9

#define	DISPLAY_RESET_DREQ			0x20

/*
**	Keyboard Characters
*/
#define	KENTER	0x0D
#define	KUP		0x1B
#define	KDOWN	0x0A
#define	KRIGHT	0x1A
#define	KLEFT	0x1C

/*
**	Macro definitions
*/
#define WATCHDOG (WDTRST = 0x1E, WDTRST = 0xE1)

#define	RMC				g_Flags0.1
#define	PANIC_UP		g_Flags0.2
#define	PANIC_DOWN		g_Flags0.3
#define	PIEZO			g_Flags0.4
#define	CLOCK_RUNNING	g_Flags0.5
#define	CLOCK_ZERO		g_Flags0.6

#define	ONE_QSECOND		g_Flags1.0
#define	ONE_SECOND		g_Flags1.1

#define	ST_PCA			g_stFlags.0
#define	ST_CHIRP		g_stFlags.1
#define	ST_RING			g_stFlags.2
#define	ST_DISPLAY		g_stFlags.3
#define	ST_GPS			g_stFlags.4
#define	ST_GSM			g_stFlags.5
#define	ST_MONITOR		g_stFlags.6
#define	ST_POF			g_stFlags.7


#define	TDRE			g_spFlags.0
#define	BREAK			g_spFlags.1

#define	gsmTDRE			g_gsmFlags.0
#define	gsmRCMP			g_gsmFlags.1
#define	gsmTXPEND		g_gsmFlags.2
#define	gsmEcho			g_gsmFlags.3
#define	gsmCall			g_gsmFlags.4

#define	gpsTDRE			g_gpsFlags.0
#define	gpsVALID		g_gpsFlags.1
#define	gpsEcho			g_gpsFlags.2

/*
**	op_state Values
*/
#define	OP_OFF				0
#define	OP_SELFTEST			1
#define	OP_RUN				2
#define	OP_PANIC_PASSIVE	3
#define	OP_PANIC_ACTIVE		4
#define	OP_FAULT			5

/*
**	lcd_state Values
*/
#define	LCD_OFF		0
#define	LCD_BLANK	1
#define	LCD_ON		2

/*
**	led_state Values
*/
#define	LED_OFF				0
#define	LED_RED				1
#define	LED_GREEN			2
#define	LED_FLASH_RED		3
#define	LED_FLASH_GREEN		4
#define	LED_RAILROAD		5

/*
**	led_input Values
*/
#define	LED_HOLD				0
#define	LED_CHANGE_OFF			(LED_OFF         + 128)
#define	LED_CHANGE_RED			(LED_RED         + 128)
#define	LED_CHANGE_GREEN		(LED_GREEN       + 128)
#define	LED_CHANGE_FLASH_RED	(LED_FLASH_RED   + 128)
#define	LED_CHANGE_FLASH_GREEN	(LED_FLASH_GREEN + 128)
#define	LED_CHANGE_RAILROAD		(LED_RAILROAD    + 128)

/*
**	piezo_state Values
*/
#define	PIEZO_OFF		0
#define	PIEZO_CHIRP		1
#define	PIEZO_ON		2

/*
**	ring vibrator states
*/
#define	RING_OFF		0
#define	RING_ON			1

/*
**	Event Definitions
*/
#define	EVENT_ON					1
#define	EVENT_OFF					2
#define	EVENT_ENTER					3
#define	EVENT_UP					4
#define	EVENT_DOWN					5
#define	EVENT_LEFT					6
#define	EVENT_RIGHT					7
#define	EVENT_PANIC					8
#define	EVENT_CANCEL_PANIC			9
#define	EVENT_CHARGER_PLUGIN		10
#define	EVENT_PANIC_DELAY_EXPIRED	11
#define	EVENT_SELFTEST_SUCCESS		12
#define	EVENT_SELFTEST_FAILURE		13

/*
**	Symbol Definitions
*/
#define	BATTERY_00		'\x03'
#define	BATTERY_20		'\x04'
#define	BATTERY_40		'\x05'
#define	BATTERY_60		'\x06'
#define	BATTERY_80		'\x07'
#define	BATTERY_100		'\x08'
/*
**	Antenna Symbol Definitions
*/
#define	ANTENNA		'\x10'
#define	_1BAR		'\x11'
#define	_2BARS		'\x12'
#define	_3BARS		'\x13'
#define	_4BARS		'\x14'
#define	_5BARS		'\x15'

#define	__OLD_GSM_MONITOR__
#undef	__OLD_GSM_MONITOR__
#define __NEW_GSM_MONITOR__

#if	defined(__NEW_GSM_MONITOR__)
#define	GSM_SLEEP			0
#define	GSM_SENDATPCMGF		1
#define	GSM_SENDATPCNMI		2
#define	GSM_SENDATPCMGS		3
#define	GSM_SENDSMSTEXT		4
#define	GSM_WAIT			5
#define	GSM_SHUTDOWN		6
#define	GSM_MODEMOFF		7
#define	GSM_SENDATPCMEE		8
#define	GSM_SENDATR			9
#define	GSM_GETCLOCK		10
#define	GSM_SETCLOCK		11
#define	GSM_AUDIOMODE		12
#define	GSM_AUDIOCHANNELS	13
#define	GSM_VOICECALL		14
#define	GSM_SIGNAL			15
#define	GSM_BATTERY			16
#define	GSM_IDLE			17
#define	GSM_NOP				18
#define	GSM_HANGUP			19
#define	GSM_SENDEMERGENCY	20
#define	GSM_SPEAKERVOLUME	21

#define	GSM_SVC_MONITOR		0
#define	GSM_SVC_PANIC		10
#define	GSM_SVC_PERIODIC	11
#define	GSM_SVC_COLDRESET	12
#define	GSM_SVC_WARMRESET	13
#define	GSM_SVC_SMSMESSAGE	14

#define	GSM_SEQ_PANIC		10
#define	GSM_SEQ_PERIODIC	11
#define	GSM_SEQ_COLDRESET	12
#define	GSM_SEQ_WARMRESET	13
#define	GSM_SEQ_SMSMESSAGE	14

#define	GSM_SUCCESS					0
#define	GSM_SEQUENCE_IN_PROGRESS	1
#define	GSM_INVALID_SERVICE			50

#elif	defined(__OLD_GSM_MONITOR__)
#define	GSM_NOP				0
#define	GSM_SENDATPCMGF		1
#define	GSM_SENDATPCNMI		2
#define	GSM_SENDATPCMGS		3
#define	GSM_SENDSMSTEXT		4
#define	GSM_WAIT			5
#define	GSM_SHUTDOWN		6
#define	GSM_MODEMOFF		7
#define	GSM_SENDATPCMEE		8
#define	GSM_SENDATR			10
#define	GSM_SETCLOCK		11
#define	GSM_AUDIOMODE		12
#define	GSM_AUDIOCHANNELS	13
#define	GSM_VOICECALL		14
#define	GSM_SIGNAL			15
#define	GSM_BATTERY			16
#define	GSM_IDLE			17
#else
#error	No GSM Monitor Selected
#endif
