/*
**  init.c
**  (c)Huron Net Works, Inc., 2005
**
**  Initialization Definition
**
**	7/5/05	--	Original
**	8/18/05 --	Refine port pin definitions for layout and first stock correction,
**				which swaps the PANIC Button with the LCD Clock so we can use Timer 2
**				to generate the Display Clock
**	9/7/05	--	Add code to determine the power up type
**
**  TABS 5, 9
*/
#include	"hardware.h"
#include	"externs.h"

/*
**	IO_INIT
**	This function is for all the I/O initialization that is required
**	as soon as possible after exiting from RESET.  The usual process
**	of clearing RAM happens after this function
*/
void	IO_INIT(void)
{
	AUXR = 0x31 ;		/* 30 clocks, 1792 Bytes, ALE only during MOVX and MOVC	*/
	CKCON0 = (1<<5)+1 ;	/* X2 Mode - Peripherals use 6 Clocks per cycle			*/
	CKCON1 = 0 ;		/* SPI uses 6 Clocks per cycle							*/
	P0 = 0xFF ;			/* Bi-directional Address/Data Bus						*/
	
	P1 = 0xFF ;		/* P1.7 thru P1.5 Reserved for SPI
					** P1.4           Piezo Output
					** P1.3           Panic Button Input
					** P1.2           Display Data Request
					** P1.1           USB Interrupt
					** P1.0           Display Clock -- T2 Output*/
					
	P2 = 0xFD ;		/* P2.7 thru P2.4 Chip Select Outputs
					** P2.3           Charger Present
					** P2.2           12V0 Shutdown, Initially Off
					** P2.1           USB 3V3 SHDN, Initially Off
					** P2.0           GPS 3V3 SHDN, Initially Off				*/
					
	P3 = 0xDF ;		/* P3.7 thru P3.6 RD* an WR*
					** P3.5           A0 for external Access - Initially Low
					** P3.4           INVALID* for R2-232
					** P3.3           GPSU_INT*
					** P3.2           GMSU_INT*
					** P3.1 thru P3.0 TXD and RXD				*/										
					
	P4 = 0xFF ;		/* P4.7           REDC
					** P4.6			  GRNC
					** P4.5           RING
					** P4.4           ACT_LEFT
					** P4.3           ACT_RIGHT
					** P4.2           ACT_DOWN
					** P4.1           ACT_UP
					** P4.0           ENTER						*/
					
	P5 = 0xF7 ;		/* P5.7           USB_RESET
					** P5.6           LCD_RESET
					** P5.5           GSM_IGT
					** P5.4           GPS_RST
					** P5.3           SPKR_SHDN
					** P5.2           GSMU_SHDN
					** P5.1           GPSU_SHDN
					** P5.0           RS232_SHDN				*/

/*
**	Initialize the Monitor Port	for 9600 Baud and N,8,1
**	Assume a 12 Mhz. Crystal and X2 Mode
**	We need SMOD1 = 1 and SPD = 1
**	BRL = 178
*/
	BRL		= 178 ;			/* 9600 Baud Constant	*/
	BDRCON	= 0x1F ;		/* Start the Baud Rate Generator and connect	**
							** it to the onboard UART						*/

/*	Start the Display Clock		*/
	T2MOD = T2OE ;
	T2CON = 0 ;				/* Select Timer Operation with Auto Reload			*/
	/* T2 & RCAP2 get -12	*/
	TH2 = RCAP2H = 0xFF ;
	TL2 = RCAP2L = 0xF4 ;
	TR2 = 1 ;				/* P1.0 should start beating up and down at this point	*/
	
/*	Do a Display RESET
**	This pulse needs to be 10 usec long
*/
	LCD_RESET = 0 ;			/* Debug instruction timing					*/
	LCD_RESET = 0 ;			/* It should be dead nuts on 10 useconds	*/
	LCD_RESET = 0 ;	
	LCD_RESET = 0 ;	
	LCD_RESET = 0 ;	
	LCD_RESET = 0 ;	
	LCD_RESET = 0 ;	
	LCD_RESET = 0 ;	
	LCD_RESET = 0 ;	
	LCD_RESET = 0 ;	
	LCD_RESET = 0 ;	
	LCD_RESET = 0 ;	
	LCD_RESET = 0 ;	
	LCD_RESET = 0 ;	
	LCD_RESET = 0 ;	
	LCD_RESET = 0 ;	
	LCD_RESET = 0 ;	
	LCD_RESET = 0 ;	
	LCD_RESET = 0 ;	
	LCD_RESET = 0 ;	
	LCD_RESET = 1 ;
	
	write_DCRI(DISPLAY_NO_ICON) ;
	write_DCRI(DISPLAY_NORMAL) ;
	write_DCRI(DISPLAY_COM_SCROLL) ;
	write_DCRI(DISPLAY_SCROLL_OFF) ;
	write_DCRI(DISPLAY_EVR+16) ;
	write_DCRI(DISPLAY_INTERNAL_PS_ON) ;

/*	Configure the UART to be a monitor port	*/
	PCON   &= POF ;					/* Power Control Register - Preserve POF	*/
	SCON	= UART8_VARIABLE ;		/* Configure Serial Port as an 8-bit UART	*/
	PCON   |= SMOD1 + SMOD0 ;		/* Make SCON.7 the Framing Error Bit		*/
									/* Use the Double Baud Rate Bit				*/
	TB8		= 1 ;					/* Make the 9th always a 1					*/
	REN		= 1 ;					/* Enable the Receiver						*/
	TI		= 1 ;					/* Fake the Initial Interrupt				*/

/*	Timer 0 and Timer 1 are used for Software Interrupts	*/
	TCON 	= 0 ;					/* Shut off both Timers										*/
	TMOD	= 0x11 ;				/* Both Timers Operate in Mode 1 - 16 bit Timer				*/
	TL0		= 0xFF ;				/* Timer 0 expires in 1 tick to synthesize Software Interrupt SWI0	*/
	TH0		= 0xFF ;
	TL1		= 0xFF ;				/* Timer 1 expires in 1 tick to synthesize Software Interrupt SWI1	*/
	TH1		= 0xFF ;

/*
**	Configure and Initialize the Programmable Counter Array
**	Mode Bits are as follows
**		CMOD.7		-- CIDL		= 0		Idle mode is not used so don't care
**		CMOD.6		-- WDTE		= 0		Using Internal Watchdog so disable
**		CMOD[2.1] 	-- CPS1..0	= 00	FPca/6 = 1.5 MHz.
**		CMOD.0		-- ECF		= 0		Counter Overflow is a don't care
*/
	CCON	= 0 ;		/* Stop Counter and Clear all Flags	*/
	CMOD	= 0 ;		/* PCA Mode Selections as above		*/
	CCAPM0	= 0 ;		/* Disable Individual Channels		*/
	CCAPM1	= 0 ;
	CCAPM2	= 0 ;
	CCAPM3	= 0 ;
	CCAPM4	= 0 ;

/*
**	SPI Initialization
**	Master Mode
**	Transmission Format 00
*/
	SPCON	 = 0 ;
	SPCON	|= MSTR ;			/* Select Master Mode				*/
	SPCON	|= SP375K000 ;		/* Select Slow Clock 375 kHz.		*/
	SPCON	|= SSDIS ;			/* Disable Slave Select function	*/
	SPCON	&= ~(CPOL + CPHA) ;	/* Transmission Format 00			*/
	SPCON	|= SPEN ;			/* SPI Enable						*/

/*
**	Keyboard Interrupt Enable
*/
	KBF		= 0 ;		/* Clear All the Keyboard Flags				*/
	KBLS	= 0 ;		/* Initialize to detect low levels			*/
	KBE		= PANIC ;	/* Enable only the PANIC Button				*/

	/*  Do a safe enable of the Keyboard Interrupt					*/
	IEN1	= (IEN1 & ESPI) | KBD ;

	CCAP1H = 0xFF ;		/*	PWM for Piezo Buzzer	*/
	CCAPM1 = 0 ;
	
	WDTPRG = 5 ;  /* set watchdog timout */
}

/*
**	LP_INIT
**	This function performs only the initialization required to operate in Low Power Mode
*/
void	LP_INIT(void)
{
/*
**	Serial Port UART Initialization
**	Keyboard Initialization
*/
	sp_Init() ;
	kbd_Init() ;
/*
**	Establish a wakeup timer
*/
#if	defined(__LP_MODE__)
	loadWakeupTimer() ;
	ES = 1 ;	/* Enable the Serial Port		*/
	EC = 1 ;	/* Enable the PCA Interrupts	*/
	CR = 1 ;	/* Start the PCA				*/
	
	EA = 1 ;	/* Global Interrupt Enable		*/
#endif		
} 
/*
**	MPU_INIT
**	This function is for any remaining I/O initialization and all non-zero
**	memory initialization.  We should not depend on the compiler to do this
**	for us.
*/
void	MPU_INIT(void)
{
/*
**	GPS UART Initialization
*/
	gps_Init() ;	
/*
**	GSM UART Initialization
*/
	gsm_Init() ;
/*
**	Initialize the key sampling shift register to
**	all keys released
*/
/*
**	Load the Tick Timer
*/
	loadTickTimer() ;
	g_flashTime		= g_Tick + 125 ;
	g_oneSecond		= g_Tick + 1000 ;
	g_keyTime		= KEY_DEBOUNCE ;		
	
	gsmTDRE = 1 ;	/* By Definition	*/
	gpsTDRE = 1 ;	/* By Definition	*/
	
	ES	= 1 ;			/* Enable Serial Port Interrupts				*/
	EC	= 1 ;			/* Enable PCA Interrupts						*/
	EX1	= 1 ;			/* External 1 is the GPS UART					*/
	EX0 = 1 ;			/* External 0 is the GSM UART					*/

	g_waitTime	= 40 ;	/* Timeout the display initialization interval	*/
	CR	= 1 ;			/* Turn on the RUN Flag							*/	

	EA	= 1 ;			/* Global Interrupt Enable						*/		
}
