/*
**  main.c
**  (c)Huron Net Works, Inc., 2005
**
**	7/1/05	-- Original
**	8/24/05	-- Add code to facilitate hardware debug and checkout
**	9/6/05	-- Add serial port interrupts
**
**  TABS 5, 9
*/
#include	"hardware.h"
#include	"externs.h"

unsigned char d_char ;
unsigned char	gc ;

extern unsigned int key_count ;

void selftest(void) ;
void set_911_Call_Delay(void) ;
void piezo_OFF(void) ;
void piezo_ON(void) ;
void piezo_Check(void) ;
void quickConfig(void) ;

#define	PARAMETER_MAX	13

#define	__DEBUG_GPS__
#undef	__DEBUG_GPS__

const unsigned char DL1[] = "ComStar Security" ;
unsigned char dlx ;

void	main	(void)
{
	LP_INIT() ;
	MPU_INIT() ;
	gps_Invalid() ;
	while(g_waitTime)	WATCHDOG ;
	ST_PCA = 1 ;		/* First Selftest Flag -- PCA is working	*/

	/* Finish the Display Initialization	*/		
	write_DCR(DISPLAY_VOLTAGE) ;
	write_DCR(DISPLAY_ON) ;
	write_DCR(DISPLAY_ADC_INVERSE) ;
	
	if(PCON & POF)	{ ST_POF = 1 ; }
	else			{ ST_POF = 0 ; }
	PCON &= ~POF ;

/*
**	Initialize the serial port monitor
*/
	spMonitorInit() ;	/* Display the banner and initialize the variables	*/

	selftest() ;

	while(led_input == LED_CHANGE_OFF) WATCHDOG ;

	led_input = LED_CHANGE_FLASH_GREEN ;
	while(led_input == LED_CHANGE_FLASH_GREEN) WATCHDOG ;

	d_clear() ;
	d_home() ;
	
	/*
	**  Each of the following disable messages is about 25 bytes
	**	All four of them represent about 100 bytes
	**	The GPS Transmit Fifo is 256 bytes long
	*/
	gps_sendDisableGGA() ;	/* Disable GGA Messages	*/
	gps_sendDisableGLL() ;	/* Disable GLL Messages	*/
	gps_sendDisableGSA() ;	/* Disable GSA Messages	*/
	gps_sendDisableGSV() ;	/* disable GSV Messages	*/

#if defined(__NEW_GSM_MONITOR__)
	gsm_state = GSM_SLEEP ;
	if(ST_POF == 1)	{	gsm_service = GSM_SVC_COLDRESET ; }
	else			{	gsm_service = GSM_SVC_WARMRESET ; }
	gsm_periodic = 1 * 60 ;	/* One minute update interval	*/
#elif defined(__OLD_GSM_MONITOR__)
	gsm_state = GSM_SENDATR ;
#else
#error	No GSM Monitor Selected
#endif
	
	EC = 0 ; g_oneSecond = g_Tick + 1000 ; ONE_SECOND = 0 ;
	EC = 1 ;
	d_tod() ;

	d_gotoxy(1,1) ;
	dlx = 0 ;
	while(DL1[dlx] != '\0') d_putch(DL1[dlx++]) ;
	
	quickConfig() ;

	while(1)
	{
		WATCHDOG ;
		if(BREAK != 0)
		{
			BREAK = 0 ;
			putch('B') ; putch('R') ; putch('K') ; putch('\a') ;
			putch('\r') ; putch('\n') ;
		}
		spMonitor() ;

		gpsMonitor() ;

		gsmMonitor() ;

		if((RMC == 1) && (gpsEcho == 0))
		{
			if(g_gpsStatus == 'V')
			{
				gps_Invalid() ;
			}
#if	defined(__DEBUG_GPS__)			
			putch(g_gpsStatus) ; putch(' ') ;
			gc = 0 ;
			while(g_latitude[gc])
			{
				putch(g_latitude[gc++]) ;
			}
			putch(' ') ;
			putch(g_latitudeIndicator) ;
			putch(' ') ;
			gc = 0 ;
			while(g_longitude[gc])
			{
				putch(g_longitude[gc++]) ;
			}
			putch(' ') ;
			putch(g_longitudeIndicator) ;
			putch(' ') ; putch('<') ; putch('-') ; putch(' ') ;
#endif			
			gc = 0 ;
			while(GPRMC[gc])
			{
				sms_GPRMC[gc] = GPRMC[gc] ;
#if	defined(__DEBUG_GPS__)
				putch(sms_GPRMC[gc++]) ;
#endif
			++gc ;				
			}
			sms_GPRMC[gc] = '\0' ;
			RMC = 0 ;
		}
		getch() ;		/* Check the keyboard	*/
		if(g_key)
		{
			switch(g_key)
			{
				case KENTER :	/*	ENTER	*/
					if(g_symbol == BATTERY_00)
					{	/* Pseudo RESET	*/
						_12V0_SHDN = 0 ;	/* Shut off the 12V supply	*/
						USB_3V3_SHDN = 0 ;	/* Shut off the USB supply	*/
						GPS_3V3_SHDN = 0 ;	/* Shut off the GPS supply	*/
						WATCHDOG ;
						EA = 0 ;	/*	Disable Interrupts						*/
						while(1) ;	/*	Allow watchdog expiration to do a RESET	*/ 
					}
					if((g_state == 0) && (g_parameter == 1))
					{
						quickConfig() ;
					}
					if((g_state == 0) && (g_parameter == 13))
					{
						gsmCall = 1 ;
						gsm_service = GSM_SVC_SMSMESSAGE ;
					}
					if((g_state == 0) && (g_parameter > 0))
					{
						d_clear() ;
						d_gotoxy(1,1) ;
						dlx = 0 ;
						while(DL1[dlx] != '\0') d_putch(DL1[dlx++]) ;
						g_parameter = 0 ;
					}
					else if(g_state == 5)
					{
						g_savedlatitudeIndicator = g_latitudeIndicator ;
						g_savedlatitude[0] = g_latitude[0] ;
						g_savedlatitude[1] = g_latitude[1] ;
						g_savedlatitude[2] = g_latitude[2] ;
						g_savedlatitude[3] = g_latitude[3] ;
						g_savedlatitude[4] = g_latitude[4] ;
						g_savedlatitude[5] = g_latitude[5] ;
						g_savedlatitude[6] = g_latitude[6] ;
						g_savedlatitude[7] = g_latitude[7] ;
						g_savedlatitude[8] = g_latitude[8] ;
						g_savedlatitude[9] = g_latitude[9] ;
						g_savedlongitudeIndicator = g_longitudeIndicator ;
						g_savedlongitude[0] = g_longitude[0] ;
						g_savedlongitude[1] = g_longitude[1] ;
						g_savedlongitude[2] = g_longitude[2] ;
						g_savedlongitude[3] = g_longitude[3] ;
						g_savedlongitude[4] = g_longitude[4] ;
						g_savedlongitude[5] = g_longitude[5] ;
						g_savedlongitude[6] = g_longitude[6] ;
						g_savedlongitude[7] = g_longitude[7] ;
						g_savedlongitude[8] = g_longitude[8] ;
						g_savedlongitude[9] = g_longitude[9] ;
						g_savedlongitude[10] = g_longitude[10] ;
					}
					break ;
				case KUP :		/*	UP		*/
					if(g_state == 0)
					{
						if(g_parameter <= 1)
						{
							g_parameter = PARAMETER_MAX ;
						}
						else
						{
							--g_parameter ;
						}
					}
					break ;
				case KDOWN :	/* DOWN		*/
								/* Go to the next attribute in the list	*/
					if(g_state == 0)
					{			
						if(g_parameter < PARAMETER_MAX)
						{
							++g_parameter ;
						}
						else
						{
							g_parameter = 1 ;
						}
					}
					break ;
				case KRIGHT :	/* RIGHT	*/
					if((g_parameter == 0) && (p_QuickConfig == 9))
					{
						g_symbol = (g_symbol + 1) & 0x7F ;
						d_gotoxy(16,1) ;
						d_putch(g_symbol) ;
					}
					else if((g_state == 0) && (g_parameter > 0))
					{
						next_alternative() ;
					}
					else
					{
						if(g_state == 0) 		g_state = 5 ;
						else if(g_state == 6)	g_state = 5 ;
						else if(g_state == 5)
						{
							d_gotoxy(1,1) ;
							dlx = 0 ;
							while(DL1[dlx] != '\0') d_putch(DL1[dlx++]) ;
							d_gotoxy(1,2) ;
							d_clear_eol() ;
						 	g_state = 0 ;
						}
					}
					break ;
				case KLEFT :	/* LEFT		*/
					if((g_parameter == 0) && (p_QuickConfig == 9))
					{
						g_symbol = (g_symbol - 1) & 0x7F ;
						d_gotoxy(16,1) ;
						d_putch(g_symbol) ;
					}
					else if((g_state == 0) && (g_parameter > 0))
					{
						prev_alternative() ;
					}
					else
					{
						if(g_state == 0)		g_state = 6 ;
						else if(g_state == 5)	g_state = 6 ; 
						else if(g_state == 6)
						{
							d_gotoxy(1,1) ;
							dlx = 0 ;
							while(DL1[dlx] != '\0') d_putch(DL1[dlx++]) ;
							d_gotoxy(1,2) ;
							d_clear_eol() ;
						 	g_state = 0 ;
						}
					}
					break ;
			}
		}
		if(PANIC_DOWN)
		{
			PANIC_DOWN = 0 ;

#if	defined(__DEBUG_PANIC_BUTTON__)
/*	Panic Button Down Debug Information	*/
			panic_down() ;
#endif
			KBLS = PANIC ;
			KBE = PANIC ;
			if(g_state == 0)		{	g_state = 1 ;	}
			else if(g_state == 1)	{	g_state = 2 ;	}
			else if(g_state == 2)
			{
				piezo_OFF() ;
				g_state = 3 ;
				gsmCall = 1 ;
			}
			else if((g_state == 3) || (g_state == 4))
			{
				piezo_OFF() ;
				d_gotoxy(1,2) ; d_clear_eol() ;	/* Clear the bottom line	*/
				g_state = 0 ;
			}
		}

		if(PANIC_UP)
		{
			PANIC_UP = 0 ;

#if	defined(__DEBUG_PANIC_BUTTON__)
/*	Panic Button Up Debug Information	*/	
			panic_up() ;
			EC = 0 ; g_int1 = g_intervalTimer ;
			EC = 1 ;
			putint(g_int1) ;
			putch('\r') ; putch('\n') ;			
#endif
			KBLS = 0 ;
			KBE = PANIC ;
		}
		if(ONE_QSECOND == 1)
		{
			ONE_QSECOND = 0 ;
			if((g_state == 0) &&(g_parameter > 0))
			{
				display_parameter() ;
			}
		}
		if(ONE_SECOND == 1)
		{
			ONE_SECOND = 0 ;
			if(gpsVALID == 1)
			{
				gpsVALID = 0 ;
				gsm_Clock[9]	= gps_Clock[0] ;
				gsm_Clock[10]	= gps_Clock[1] ;
				
				gps_hours = ((gsm_Clock[9] - '0') * 10) + (gsm_Clock[10] - '0') ;
				gps_hours = gps_hours - 5 ;
				if(gps_hours > 250)	gps_hours = gps_hours + 24 ;
				gsm_Clock[9] 	= (gps_hours / 10) + '0' ;
				gsm_Clock[10]	= (gps_hours % 10) + '0' ;
				
				gsm_Clock[11]	= ':' ;
				gsm_Clock[12]	= gps_Clock[2] ;
				gsm_Clock[13]	= gps_Clock[3] ;
				gsm_Clock[14]	= ':' ;
				gsm_Clock[15]	= gps_Clock[4] ;
				gsm_Clock[16]	= gps_Clock[5] ;
			}
			else
			{
				d_tod_plus1() ;
			}
			switch(g_state)
			{
				case 0 :
					if(g_parameter == 0)
					{
						d_tod() ;
						d_batteryCapacity() ;
						d_signalQuality() ;
						d_gotoxy(16,2) ; d_putch(g_gpsStatus) ;
#if	defined(__NEW_GSM_MONITOR__)
						if(gsm_periodic == 0)
						{
							if(gsm_status == 0)
							{
								gsm_service = GSM_SVC_PERIODIC ;
								gsm_periodic = 1 * 60 ;	/* One Minute Update Interval	*/
							}
						}
#endif	
					}
					break ;
				case 1 :
					/* Initialize the clock	*/
					if(GSM_IGT == 1) GSM_IGT = 0 ;
					
					set_911_Call_Delay() ;
					g_state = 2 ;
					break ;
				case 2 :
					/* Countdown Clock	*/
					d_clock_minus1() ;
					if(CLOCK_ZERO == 0) break ;
					gsmCall = 1 ;
					nCall = 0 ;
					gsm_service = GSM_SVC_PANIC ;
					CLOCK_RUNNING = 0 ;
					g_state = 3 ;
					piezo_ON() ;
				case 3 :
					piezo_Check() ;
					d_gotoxy(1,2) ;
					d_latitude() ;
					g_state = 4 ;
					if(gsmCall == 0)
					{
						piezo_OFF() ;
						g_state = 0 ;
						d_gotoxy(1,2) ;
						d_clear_eol() ;
						d_tod() ;
					}				
					break ;
				case 4 :
					piezo_Check() ;
					d_gotoxy(1,2) ;
					d_longitude() ;
					g_state = 3 ;
					if(gsmCall == 0)
					{
						piezo_OFF() ;
						g_state = 0 ;
						d_gotoxy(1,2) ;
						d_clear_eol() ;
						d_tod() ;
					}
					break ;
				case 5 :
					d_gotoxy(1,1) ;
					d_latitude() ;
					d_gotoxy(1,2) ;
					d_longitude() ;
					break ;
				case 6 :
					d_gotoxy(1,1) ;
					d_savedlatitude() ;
					d_gotoxy(1,2) ;
					d_savedlongitude() ;
					break ;	
				default :
					g_state = 0 ;
					break ;
			}
		}	
	}		/* END while(1)				*/
}			/* END main					*/

void	selftest(void)
{
	op_state = OP_SELFTEST ;

	EC = 0 ; g_ringTime = 500 ;
	EC = 1 ;
	
	RING = 0 ;
	
	g_chirpTime = 100 ;
	g_arg1 = CCAP1H ;	/* Save the current duty cycle value		*/
	CCAP1H = 0xF0 ;		/* Set the chirp duty cycle value			*/
	CCAPM1 = 0x42 ;		/* Configure PCA channel #1 for PWM Mode	*/
	PIEZO = 1 ;
	
	d_clear() ;

	led_input = LED_CHANGE_RED ;
	d_home() ;
	while(led_input == LED_CHANGE_RED) WATCHDOG ;

	led_input = LED_CHANGE_GREEN ;
	d_putch('S') ; d_putch('e') ; d_putch('l') ; d_putch('f') ;
	d_putch('t') ; d_putch('e') ; d_putch('s') ; d_putch('t') ;
	while(led_input == LED_CHANGE_GREEN) WATCHDOG ;

	led_input = LED_CHANGE_OFF ;
	d_gotoxy(1,2) ;
	d_char = '|' ;
	d_putch(d_char) ;
	g_waitTime = 250 ;
	
	while(op_state == OP_SELFTEST)
	{
		WATCHDOG ;
		if(g_waitTime == 0)
		{
			switch(d_char)
			{
				case '|'	: 	d_char = '/' ;	break ;
				case '/'	: 	d_char = '-' ;	break ;
				case '-'	: 	d_char = '\\' ;	break ;
				case '\\'	:
				default		: 	d_char = '|' ;
								ST_DISPLAY = 1 ;
								break ;	
			}
			g_waitTime = 250 ;
			d_gotoxy(1,2) ;
			d_putch(d_char) ;
		}
		if(PIEZO == 1)
		{
			if(g_chirpTime == 0)
			{
				CCAPM1 = 0 ;		/* Disable PCA channel #1					*/
				CCAP1H = g_arg1 ;	/* Restore the previous duty cycle value	*/
				PIEZO = 0 ;
				ST_CHIRP = 1 ;
			}
		}
		EC = 0 ; if(g_ringTime == 0)
		{
			RING = 1 ;
			ST_RING = 1 ;
		}
		EC = 1 ;
		if(gsm_nRx > 0)	ST_GSM = 1 ;
		if(gps_nRx > 0)	ST_GPS = 1 ;
		if(sp_nRx  > 0)	ST_MONITOR = 1 ;
		if((g_stFlags & 15)== 15)
		{	/* Basic Tasks are complete			*/
			op_state = OP_RUN ;
		}
	}
}

void	set_911_Call_Delay(void)
{
	g_clock[0] = '0' ; g_clock[1] = '0' ;
	g_clock[2] = ':' ;
	switch(p_911CallDelay)
	{
		default: p_911CallDelay = 1 ;
		case 1 :	g_clock[3] = '0' ; g_clock[4] = '0' ; break ;
		case 2 :	g_clock[3] = '0' ; g_clock[4] = '5' ; break ;
		case 3 :	g_clock[3] = '1' ; g_clock[4] = '0' ; break ;
		case 4 :	g_clock[3] = '1' ; g_clock[4] = '5' ; break ;
		case 5 :	g_clock[1] = '1' ; g_clock[3] = '0' ; g_clock[4] = '0' ; break ;
		case 6 :	g_clock[3] = '0' ; g_clock[4] = '0' ; break ;
	}					
	d_clock() ;
	CLOCK_ZERO = 0 ;
	CLOCK_RUNNING = 1 ;
}

void	piezo_OFF(void)
{
	CCAPM1 = 0 ;		/* Disable PCA channel #1							*/
	CCAP1H = g_arg1 ;	/* Restore the previous duty cycle value			*/
	PIEZO = 0 ;
}

void	piezo_ON(void)
{
	if(p_PanicAlarmTime == 5) return ;
	/* Turn the PIEZO on when we enter this state					*/
	g_arg1 = CCAP1H ;	/* Save the current duty cycle value		*/
	switch(p_PanicAlarmSound)
	{
		default:	p_PanicAlarmSound = 1 ;
		case 1 :	CCAP1H = 0xF8 ;		CCAPM1 = 0x42 ;	PIEZO = 1 ; break ;
		case 2 :	CCAP1H = 0x7C ;		CCAPM1 = 0x42 ; PIEZO = 1 ; break ;
		case 3 :	CCAP1H = g_arg1 ; 	CCAPM1 = 0    ; PIEZO = 0 ; break ;
	}
	
	EC = 0 ; PA_Timer = 0 ;
		switch(p_PanicAlarmRepeat)
		{
			default :	p_PanicAlarmRepeat = 1 ;
			case 1 :	PA_Repeat =  1 * 15 ;	break ;
			case 2 :	PA_Repeat =  1 * 30 ;	break ;
			case 3 :	PA_Repeat =  1 * 60 ;	break ;
			case 4 :	PA_Repeat =  5 * 60 ;	break ;
			case 5 :	PA_Repeat = 15 * 60 ;	break ;
			case 6 :	PA_Repeat =  0 ;		break ;
		}
	EC = 1 ;
}

void	piezo_Check(void)
{
	switch(p_PanicAlarmTime)
	{
		default :	p_PanicAlarmTime = 1 ;
		case 1 :	EC = 0 ; if(PA_Timer > 2000)	{ piezo_OFF() ;} EC = 1 ; break ;
		case 2 :	EC = 0 ; if(PA_Timer > 5000)	{ piezo_OFF() ;} EC = 1 ; break ;
		case 3 :	EC = 0 ; if(PA_Timer > 10000)	{ piezo_OFF() ;} EC = 1 ; break ;
		case 4 :	break ;					/* Continuous	*/
		case 5 :	piezo_OFF() ; break ;	/* Disable		*/
	}
	if(p_PanicAlarmRepeat == 6) return ;
	EC = 0 ; if(PA_Repeat != 0) { EC = 1 ; return ; }
	EC = 1 ;
	piezo_ON() ;
}

void	quickConfig(void)
{
/*	Panic Alarm Time	*/
	switch(p_QuickConfig)
	{
		default :	p_QuickConfig = 1 ;
		case 1 :	
		case 2 :
		case 4 :
		case 6 :
		case 7 :
		case 8 :	p_PanicAlarmTime = 2 ;	/* 5 Seconds	*/
					break ;
		case 5 :	p_PanicAlarmTime = 1 ;	/* 2 Seconds	*/
					break ;
		case 3 :	p_PanicAlarmTime = 4 ;	/* Continuous	*/
		case 9 :	break ;
	}
/*	Panic Alarm Repeat	*/		
	switch(p_QuickConfig)
	{
		default :	p_QuickConfig = 1 ;
		case 1 :	
		case 2 :
		case 4 :
		case 5 :
		case 6 :
		case 7 :
		case 8 :	p_PanicAlarmRepeat = 1 ;	/* 15 Seconds	*/
					break ;
		case 3 :	p_PanicAlarmRepeat = 6 ;	/* Disable		*/
		case 9 :	break ;
	}
/*	Panic Alarm Sound	*/
	switch(p_QuickConfig)
	{
		default :	p_QuickConfig = 1 ;
		case 1 :	
		case 3 :
		case 5 :
		case 6 :
		case 7 :
		case 8 :	p_PanicAlarmSound = 2 ;	/* High			*/
					break ;
		case 4 :	p_PanicAlarmSound = 1 ;	/* Low			*/
					break ;			
		case 2 :	p_PanicAlarmSound = 3 ;	/* Disable		*/
		case 9 :	break ;
	}
/*	911 Call Delay	*/
	switch(p_QuickConfig)
	{
		default :	p_QuickConfig = 1 ;
		case 1 :
		case 3 :	p_911CallDelay = 1 ;	/* 0 seconds	*/
					break ;
		case 2 :
		case 6 :
		case 7 :
		case 8 :	p_911CallDelay = 2 ;	/* 5 Seconds	*/
					break ;
		case 4 :	p_911CallDelay = 4 ;	/* 15 seconds	*/
					break ;
		case 5 :	p_911CallDelay = 6 ;	/* Disable		*/
		case 9 :	break ;	
	}
/*	Speaker Volume	*/
	switch(p_QuickConfig)
	{
		default :	p_QuickConfig = 1 ;
		case 1 :
		case 3 :
		case 4 :
		case 6 :
		case 7 :
		case 8 :	p_SpeakerVolume = 2 ;	/* Medium	*/
					break ;
		case 2 :	p_SpeakerVolume = 1 ;	/* Low		*/
					break ;
		case 5 :	p_SpeakerVolume = 3 ;	/* High		*/
		case 9 :	break ; 	
	}
/*	GPS Report	*/
	switch(p_QuickConfig)
	{
		default :	p_QuickConfig = 1 ;
		case 1 :
		case 2 :
		case 3 :
		case 5 :
		case 6 :
		case 7 :
		case 8 :	p_GPSReport = 1 ;	/* 911 Call Only	*/
					break ;
		case 4 :	p_GPSReport = 3 ;	/* 5 minutes		*/
		case 9 :	break ;
	}
/*	GPS Power	*/
	switch(p_QuickConfig)
	{
		default :	p_QuickConfig = 1 ;
		case 1 :
		case 2 :
		case 3 :
		case 5 :
		case 6 :
		case 7 :
		case 8 :	p_GPSPower = 2 ;	/* Standby		*/
					break ;
		case 4 :	p_GPSPower = 1 ;	/* ON			*/
		case 9 :	break ;
	}
/*	SMSService		*/
	switch(p_QuickConfig)
	{
		default :	p_QuickConfig = 1 ;
		case 1 :
		case 2 :
		case 3 :
		case 4 :
		case 5 :
		case 6 :
		case 7 :
		case 8 :	p_SMSService = 1 ;	/* Default		*/
		case 9 :	break ;
	}
/*	SMSAlert		*/
	switch(p_QuickConfig)
	{
		default :	p_QuickConfig = 1 ;
		case 1 :
		case 3 :
		case 4 :
		case 5 :
		case 6 :
		case 7 :
		case 8 :	p_SMSAlert = 3 ;	/* Vibrate & Sound	*/
					break ;
		case 2 :	p_SMSAlert = 1 ;	/* Vibrate			*/
		case 9 :	break ;
	}
}

#if	0
void	main(void)
{
	selftest() ;
	while(1)
	{
		WATCHDOG ;
		switch(op_state)
		{
			case OP_OFF :
			case OP_SELFTEST :
			case OP_RUN :
			case OP_PANIC_PASSIVE :
			case OP_PANIC_ACTIVE :
			case OP_FAULT :
			default:
		}
	}
}
#endif

